%James Hing
%Matlab Tutorial
%Voronoi Diagrams
%This code demonstrates the perpedicular bisector incremental construction approach for generating 
%voronoi diagrams.


%Clears any previously stored variables in matlab
clear;clear all;

%Asks the user to enter in the length of the side for a square workspace
window = input('Enter the window length (square): ');

%Asks the user to input the number of points to generate a voronoi diagram (currently only works for 3 points)
numbpoints = input('Enter the number of points:  ');

%Initializes the Workspace window
figure(1)
axis([0 window 0 window])
grid on;
hold on;
xlabel('Select points with left mouse button,to select the last obstacle use the Right button','Color','blue');
but=1;

%Waits for the User to click on the desired location for the points
while but == 1
    for count=1:1:numbpoints
        [xpos(count),ypos(count),but] = ginput(1);
        plot(xpos,ypos,'ro');
    end
 end%End of While loop

 for n=1:1:numbpoints
     Coord{n}=[xpos(n) ypos(n)];
 end
 temp=1;
  for i=2:1:numbpoints
      for j=1:1:i-1
          %Calculates the equation of line between points
          m(temp) = (Coord{i}(2)-Coord{j}(2))/(Coord{i}(1)-Coord{j}(1));
          mb(temp) = -1/m(temp);
          %Calcuates the midpoint of the line
          midy(temp) = (Coord{i}(2)+Coord{j}(2))/2;
          midx(temp) = (Coord{i}(1)+Coord{j}(1))/2;
%Point where perpendicular bisector crosses the window axes
    D(temp) = midy(temp)-midx(temp)*mb(temp);
    TOP(temp) = (window-D(temp))/mb(temp);
    LEFT(temp) = D(temp);
    RIGHT(temp) = mb(temp)*window+D(temp);
    BOTTOM(temp) = -D(temp)/mb(temp);
%Solve for bisector intersections
    xbisec(1)=0;
    ybisce(1)=0;
    xbisec(2)=0;
    ybisec(2)=0;


%The following code sets up the constraints for plotting on the workspace
if TOP(temp)<=window & TOP(temp)>=0 & BOTTOM(temp)<=window & BOTTOM(temp)>=0
    xbisec(1)=TOP(temp);
    ybisec(1)=window;
    xbisec(2)=BOTTOM(temp);
    ybisec(2)=0;
end

if RIGHT(temp)<window & RIGHT(temp)>0 & LEFT(temp)<window & LEFT(temp)>0
    xbisec(1)=0;
    ybisec(1)=LEFT(temp);
    xbisec(2)=window;
    ybisec(2)=RIGHT(temp);
end

if TOP(temp)<window & TOP(temp)>0 & LEFT(temp)<window & LEFT(temp)>0
    xbisec(1)=0;
    ybisec(1)=LEFT(temp);
    xbisec(2)=TOP(temp);
    ybisec(2)=window;
end

if TOP(temp)<window & TOP(temp)>0 & RIGHT(temp)<window & RIGHT(temp)>0
    xbisec(1)=TOP(temp);
    ybisec(1)=window;
    xbisec(2)=window;
    ybisec(2)=RIGHT(temp);
end

if BOTTOM(temp)<window & BOTTOM(temp)>0 & RIGHT(temp)<window & RIGHT(temp)>0
    xbisec(1)=BOTTOM(temp);
    ybisec(1)=0;
    xbisec(2)=window;
    ybisec(2)=RIGHT(temp);
end

if BOTTOM(temp)<window & BOTTOM(temp)>0 & LEFT(temp)<window & LEFT(temp)>0
    xbisec(1)=0;
    ybisec(1)=LEFT(temp);
    xbisec(2)=BOTTOM(temp);
    ybisec(2)=0;
end

    bisecx{temp}(1) = xbisec(1);
    linepoint1x(temp) = bisecx{temp}(1);
    bisecx{temp}(2) = xbisec(2);
    linepoint2x(temp) = bisecx{temp}(2);
    bisecy{temp}(1) = ybisec(1);
    linepoint1y(temp)= bisecy{temp}(1);
    bisecy{temp}(2) = ybisec(2);
    linepoint2y(temp)= bisecy{temp}(2);

%Plots the perpendicular bisectors
    plot(bisecx{temp},bisecy{temp},'b',xpos,ypos,'x',xpos,ypos,'o');
axis([0 window 0 window])
temp=temp+1;
end
end


%Finding the intersection between the perpendicular bisectors of each point (intersection becomes a voronoi vertex)
count1=1; 
for q=2:1:temp-1
    for p=1:1:q-1
        c=0;
        d=0;
        e=0;
        f=0;
        %Calculate equation of the lines that are intersecting
         slope1 = ((linepoint2y(q)-linepoint1y(q))/(linepoint2x(q)-linepoint1x(q)));
         slope2 = ((linepoint2y(p)-linepoint1y(p))/(linepoint2x(p)-linepoint1x(p)));
         B1 = linepoint2y(q)-slope1*linepoint2x(q);
         B2 = linepoint2y(p)-slope2*linepoint2x(p);
         %Calculation of the intersection point between the two lines
         xintercept=(B2-B1)/(slope1-slope2);
         yintercept=slope2*xintercept+B2;
         
         %Calculating the distance of the end points of the bisector lines to each site on the plane 
         for zz=1:1:numbpoints
             distance1{zz}(1)=sqrt((linepoint1x(q)-Coord{zz}(1))^2+(linepoint1y(q)-Coord{zz}(2))^2);
             dismin11(zz)=round(distance1{zz}(1)*1000);
             distance1{zz}(2)=sqrt((linepoint2x(q)-Coord{zz}(1))^2+(linepoint2y(q)-Coord{zz}(2))^2);
             dismin12(zz)=round(distance1{zz}(2)*1000);
             distance2{zz}(1)=sqrt((linepoint1x(p)-Coord{zz}(1))^2+(linepoint1y(p)-Coord{zz}(2))^2);
             dismin21(zz)=round(distance2{zz}(1)*1000);
             distance2{zz}(2)=sqrt((linepoint2x(p)-Coord{zz}(1))^2+(linepoint2y(p)-Coord{zz}(2))^2);
             dismin22(zz)=round(distance2{zz}(2)*1000);
        end        
        distancemin11 = min(dismin11);
        distancemin12 = min(dismin12);
        distancemin21 = min(dismin21);
        distancemin22 = min(dismin22);
        dm11=sort(dismin11);
        dm12=sort(dismin12);
        dm21=sort(dismin21);
        dm22=sort(dismin22);
        
        %Determining which section of the perpendicular bisectors violate the voronoi rules and this section
        %of the bisector will be eliminated from the graph
        for y=1:1:numbpoints
            if dm11(y)==distancemin11
                c=c+1;
            end
            if dm12(y)==distancemin12
                d=d+1;
            end
            if dm21(y)==distancemin21
                e=e+1;
            end
            if dm22(y)==distancemin22
                f=f+1;
            end
        end
        if c < 2
            linepoint1x(q)=xintercept;
            linepoint1y(q)=yintercept;
        end
        if d < 2
            linepoint2x(q)=xintercept;
            linepoint2y(q)=yintercept;
        end
        if e < 2
            linepoint1x(p)=xintercept;
            linepoint1y(p)=yintercept;
        end
        if f < 2
            linepoint2x(p)=xintercept;
            linepoint2y(p)=yintercept;
        end
    end
end


%PLOTTING THE FINAL VORONOI DIAGRAM
figure(3)
hold on;
for w=1:1:temp-1
plot([linepoint1x(w) linepoint2x(w)],[linepoint1y(w) linepoint2y(w)],'b',xpos,ypos,'x',xpos,ypos,'o');
axis([0 window 0 window])
end

